@@ -1,8 +1,6 @@
-# encoding: utf-8
-
 # The MIT License
 #
-# Copyirhgt (c) 2015 Sebastian Ramacher
+# Copyright (c) 2015 Sebastian Ramacher
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -22,8 +20,10 @@
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 # THE SOFTWARE.
 
+
 import sys
 
-if __name__ == '__main__':
-    from bpython.curtsies import main
+if __name__ == "__main__":
+    from .curtsies import main
+
     sys.exit(main())
