@@ -1,8 +1,7 @@
-# coding: utf-8
-
 # The MIT License
 #
 # Copyright (c) 2009-2015 the bpython authors.
+# Copyright (c) 2015-2020 Sebastian Ramacher
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -21,9 +20,7 @@
 # LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 # THE SOFTWARE.
-#
 
-from __future__ import unicode_literals
 
 import __main__
 import abc
@@ -33,46 +30,138 @@ import logging
 import os
 import re
 import rlcompleter
-import sys
-from six.moves import range, builtins
-from six import string_types, iteritems
+import builtins
 
-from bpython import inspection
-from bpython import importcompletion
-from bpython import line as lineparts
-from bpython.line import LinePart
-from bpython._py3compat import py3, try_decode
-from bpython.lazyre import LazyReCompile
-from bpython.simpleeval import (safe_eval, evaluate_current_expression,
-                                EvaluationError)
-
-if not py3:
-    from types import InstanceType, ClassType
+from enum import Enum
+from . import inspection
+from . import line as lineparts
+from .line import LinePart
+from .lazyre import LazyReCompile
+from .simpleeval import safe_eval, evaluate_current_expression, EvaluationError
 
 
 # Autocomplete modes
-SIMPLE = 'simple'
-SUBSTRING = 'substring'
-FUZZY = 'fuzzy'
+class AutocompleteModes(Enum):
+    SIMPLE = "simple"
+    SUBSTRING = "substring"
+    FUZZY = "fuzzy"
 
-ALL_MODES = (SIMPLE, SUBSTRING, FUZZY)
+    @classmethod
+    def from_string(cls, value):
+        if value in cls.__members__:
+            return cls.__members__[value]
+        return None
 
-MAGIC_METHODS = tuple("__%s__" % s for s in (
-    "init", "repr", "str", "lt", "le", "eq", "ne", "gt", "ge", "cmp", "hash",
-    "nonzero", "unicode", "getattr", "setattr", "get", "set", "call", "len",
-    "getitem", "setitem", "iter", "reversed", "contains", "add", "sub", "mul",
-    "floordiv", "mod", "divmod", "pow", "lshift", "rshift", "and", "xor", "or",
-    "div", "truediv", "neg", "pos", "abs", "invert", "complex", "int", "float",
-    "oct", "hex", "index", "coerce", "enter", "exit"))
 
-if py3:
-    KEYWORDS = frozenset(keyword.kwlist)
-else:
-    KEYWORDS = frozenset(name.decode('ascii') for name in keyword.kwlist)
+MAGIC_METHODS = tuple(
+    f"__{s}__"
+    for s in (
+        "new",
+        "init",
+        "del",
+        "repr",
+        "str",
+        "bytes",
+        "format",
+        "lt",
+        "le",
+        "eq",
+        "ne",
+        "gt",
+        "ge",
+        "hash",
+        "bool",
+        "getattr",
+        "getattribute",
+        "setattr",
+        "delattr",
+        "dir",
+        "get",
+        "set",
+        "delete",
+        "set_name",
+        "init_subclass",
+        "instancecheck",
+        "subclasscheck",
+        "class_getitem",
+        "call",
+        "len",
+        "length_hint",
+        "getitem",
+        "setitem",
+        "delitem",
+        "missing",
+        "iter",
+        "reversed",
+        "contains",
+        "add",
+        "sub",
+        "mul",
+        "matmul",
+        "truediv",
+        "floordiv",
+        "mod",
+        "divmod",
+        "pow",
+        "lshift",
+        "rshift",
+        "and",
+        "xor",
+        "or",
+        "radd",
+        "rsub",
+        "rmul",
+        "rmatmul",
+        "rtruediv",
+        "rfloordiv",
+        "rmod",
+        "rdivmod",
+        "rpow",
+        "rlshift",
+        "rrshift",
+        "rand",
+        "rxor",
+        "ror",
+        "iadd",
+        "isub",
+        "imul",
+        "imatmul",
+        "itruediv",
+        "ifloordiv",
+        "imod",
+        "ipow",
+        "ilshift",
+        "irshift",
+        "iand",
+        "ixor",
+        "ixor",
+        "neg",
+        "pos",
+        "abs",
+        "invert",
+        "complex",
+        "int",
+        "float",
+        "index",
+        "round",
+        "trunc",
+        "floor",
+        "ceil",
+        "enter",
+        "exit",
+        "await",
+        "aiter",
+        "anext",
+        "aenter",
+        "aexit",
+    )
+)
+
+KEYWORDS = frozenset(keyword.kwlist)
 
 
 def after_last_dot(name):
-    return name.rstrip('.').rsplit('.')[-1]
+    return name.rstrip(".").rsplit(".")[-1]
 
 
 def few_enough_underscores(current, match):
@@ -82,11 +171,11 @@ def few_enough_underscores(current, match):
     if current is __, True regardless of match
     otherwise True if match does not start with any underscore
     """
-    if current.startswith('__'):
+    if current.startswith("__"):
         return True
-    elif current.startswith('_') and not match.startswith('__'):
+    elif current.startswith("_") and not match.startswith("__"):
         return True
-    elif match.startswith('_'):
+    elif match.startswith("_"):
         return False
     else:
         return True
@@ -101,21 +190,21 @@ def method_match_substring(word, size, text):
 
 
 def method_match_fuzzy(word, size, text):
-    s = r'.*%s.*' % '.*'.join(list(text))
+    s = r".*%s.*" % ".*".join(list(text))
     return re.search(s, word)
 
 
 MODES_MAP = {
-    SIMPLE: method_match_simple,
-    SUBSTRING: method_match_substring,
-    FUZZY: method_match_fuzzy
+    AutocompleteModes.SIMPLE: method_match_simple,
+    AutocompleteModes.SUBSTRING: method_match_substring,
+    AutocompleteModes.FUZZY: method_match_fuzzy,
 }
 
 
-class BaseCompletionType(object):
+class BaseCompletionType:
     """Describes different completion types"""
 
-    def __init__(self, shown_before_tab=True, mode=SIMPLE):
+    def __init__(self, shown_before_tab=True, mode=AutocompleteModes.SIMPLE):
         self._shown_before_tab = shown_before_tab
         self.method_match = MODES_MAP[mode]
 
@@ -134,7 +223,7 @@ class BaseCompletionType(object):
             * determine whether suggestions should be `shown_before_tab`
             * `substitute(cur, line, match)` in a match for what's found with
               `target`
-            """
+        """
         raise NotImplementedError
 
     def locate(self, cursor_offset, line):
@@ -152,7 +241,7 @@ class BaseCompletionType(object):
         """Returns a cursor offset and line with match swapped in"""
         lpart = self.locate(cursor_offset, line)
         offset = lpart.start + len(match)
-        changed_line = line[:lpart.start] + match + line[lpart.end:]
+        changed_line = line[: lpart.start] + match + line[lpart.end :]
         return offset, changed_line
 
     @property
@@ -165,13 +254,14 @@ class BaseCompletionType(object):
 class CumulativeCompleter(BaseCompletionType):
     """Returns combined matches from several completers"""
 
-    def __init__(self, completers, mode=SIMPLE):
+    def __init__(self, completers, mode=AutocompleteModes.SIMPLE):
         if not completers:
             raise ValueError(
-                "CumulativeCompleter requires at least one completer")
+                "CumulativeCompleter requires at least one completer"
+            )
         self._completers = completers
 
-        super(CumulativeCompleter, self).__init__(True, mode)
+        super().__init__(True, mode)
 
     def locate(self, current_offset, line):
         return self._completers[0].locate(current_offset, line)
@@ -183,9 +273,9 @@ class CumulativeCompleter(BaseCompletionType):
         return_value = None
         all_matches = set()
         for completer in self._completers:
-            matches = completer.matches(cursor_offset=cursor_offset,
-                                        line=line,
-                                        **kwargs)
+            matches = completer.matches(
+                cursor_offset=cursor_offset, line=line, **kwargs
+            )
             if matches is not None:
                 all_matches.update(matches)
                 return_value = all_matches
@@ -194,9 +284,12 @@ class CumulativeCompleter(BaseCompletionType):
 
 
 class ImportCompletion(BaseCompletionType):
+    def __init__(self, module_gatherer, mode=AutocompleteModes.SIMPLE):
+        super().__init__(False, mode)
+        self.module_gatherer = module_gatherer
 
     def matches(self, cursor_offset, line, **kwargs):
-        return importcompletion.complete(cursor_offset, line)
+        return self.module_gatherer.complete(cursor_offset, line)
 
     def locate(self, current_offset, line):
         return lineparts.current_word(current_offset, line)
@@ -206,20 +299,11 @@ class ImportCompletion(BaseCompletionType):
 
 
 class FilenameCompletion(BaseCompletionType):
+    def __init__(self, mode=AutocompleteModes.SIMPLE):
+        super().__init__(False, mode)
 
-    def __init__(self, mode=SIMPLE):
-        super(FilenameCompletion, self).__init__(False, mode)
-
-    if sys.version_info[:2] >= (3, 4):
-        def safe_glob(self, pathname):
-            return glob.iglob(glob.escape(pathname) + '*')
-    else:
-        def safe_glob(self, pathname):
-            try:
-                return glob.glob(pathname + '*')
-            except re.error:
-                # see #491
-                return tuple()
+    def safe_glob(self, pathname):
+        return glob.iglob(glob.escape(pathname) + "*")
 
     def matches(self, cursor_offset, line, **kwargs):
         cs = lineparts.current_string(cursor_offset, line)
@@ -231,8 +315,8 @@ class FilenameCompletion(BaseCompletionType):
         for filename in self.safe_glob(os.path.expanduser(cs.word)):
             if os.path.isdir(filename):
                 filename += os.path.sep
-            if cs.word.startswith('~'):
-                filename = username + filename[len(user_dir):]
+            if cs.word.startswith("~"):
+                filename = username + filename[len(user_dir) :]
             matches.add(filename)
         return matches
 
@@ -242,7 +326,7 @@ class FilenameCompletion(BaseCompletionType):
     def format(self, filename):
         filename.rstrip(os.sep).rsplit(os.sep)[-1]
         if os.sep in filename[:-1]:
-            return filename[filename.rindex(os.sep, 0, -1) + 1:]
+            return filename[filename.rindex(os.sep, 0, -1) + 1 :]
         else:
             return filename
 
@@ -252,9 +336,9 @@ class AttrCompletion(BaseCompletionType):
     attr_matches_re = LazyReCompile(r"(\w+(\.\w+)*)\.(\w*)")
 
     def matches(self, cursor_offset, line, **kwargs):
-        if 'locals_' not in kwargs:
+        if "locals_" not in kwargs:
             return None
-        locals_ = kwargs['locals_']
+        locals_ = kwargs["locals_"]
 
         r = self.locate(cursor_offset, line)
         if r is None:
@@ -263,19 +347,23 @@ class AttrCompletion(BaseCompletionType):
         if locals_ is None:  # TODO add a note about why
             locals_ = __main__.__dict__
 
-        assert '.' in r.word
+        assert "." in r.word
 
         for i in range(1, len(r.word) + 1):
-            if r.word[-i] == '[':
+            if r.word[-i] == "[":
                 i -= 1
                 break
         methodtext = r.word[-i:]
-        matches = set(''.join([r.word[:-i], m])
-                      for m in self.attr_matches(methodtext, locals_))
+        matches = {
+            "".join([r.word[:-i], m])
+            for m in self.attr_matches(methodtext, locals_)
+        }
 
-        return set(m for m in matches
-                   if few_enough_underscores(r.word.split('.')[-1],
-                                             m.split('.')[-1]))
+        return {
+            m
+            for m in matches
+            if few_enough_underscores(r.word.split(".")[-1], m.split(".")[-1])
+        }
 
     def locate(self, current_offset, line):
         return lineparts.current_dotted_attribute(current_offset, line)
@@ -284,12 +372,8 @@ class AttrCompletion(BaseCompletionType):
         return after_last_dot(word)
 
     def attr_matches(self, text, namespace):
-        """Taken from rlcompleter.py and bent to my will.
-        """
+        """Taken from rlcompleter.py and bent to my will."""
 
-        # Gna, Py 2.6's rlcompleter searches for __call__ inside the
-        # instance instead of the type, so we monkeypatch to prevent
-        # side-effects (__getattr__/__getattribute__)
         m = self.attr_matches_re.match(text)
         if not m:
             return []
@@ -303,58 +387,41 @@ class AttrCompletion(BaseCompletionType):
             obj = safe_eval(expr, namespace)
         except EvaluationError:
             return []
-        with inspection.AttrCleaner(obj):
-            matches = self.attr_lookup(obj, expr, attr)
+        matches = self.attr_lookup(obj, expr, attr)
         return matches
 
     def attr_lookup(self, obj, expr, attr):
-        """Second half of original attr_matches method factored out so it can
-        be wrapped in a safe try/finally block in case anything bad happens to
-        restore the original __getattribute__ method."""
+        """Second half of attr_matches."""
         words = self.list_attributes(obj)
-        if hasattr(obj, '__class__'):
-            words.append('__class__')
-            words = words + rlcompleter.get_class_members(obj.__class__)
-            if not isinstance(obj.__class__, abc.ABCMeta):
+        if inspection.hasattr_safe(obj, "__class__"):
+            words.append("__class__")
+            klass = inspection.getattr_safe(obj, "__class__")
+            words = words + rlcompleter.get_class_members(klass)
+            if not isinstance(klass, abc.ABCMeta):
                 try:
-                    words.remove('__abstractmethods__')
+                    words.remove("__abstractmethods__")
                 except ValueError:
                     pass
 
-        if not py3 and isinstance(obj, (InstanceType, ClassType)):
-            # Account for the __dict__ in an old-style class.
-            words.append('__dict__')
-
         matches = []
         n = len(attr)
         for word in words:
             if self.method_match(word, n, attr) and word != "__builtins__":
-                matches.append("%s.%s" % (expr, word))
+                matches.append(f"{expr}.{word}")
         return matches
 
-    if py3:
-        def list_attributes(self, obj):
+    def list_attributes(self, obj):
+        # TODO: re-implement dir using getattr_static to avoid using
+        # AttrCleaner here?
+        with inspection.AttrCleaner(obj):
             return dir(obj)
-    else:
-        def list_attributes(self, obj):
-            if isinstance(obj, InstanceType):
-                try:
-                    return dir(obj)
-                except Exception:
-                    # This is a case where we can not prevent user code from
-                    # running. We return a default list attributes on error
-                    # instead. (#536)
-                    return ['__doc__', '__module__']
-            else:
-                return dir(obj)
 
 
 class DictKeyCompletion(BaseCompletionType):
-
     def matches(self, cursor_offset, line, **kwargs):
-        if 'locals_' not in kwargs:
+        if "locals_" not in kwargs:
             return None
-        locals_ = kwargs['locals_']
+        locals_ = kwargs["locals_"]
 
         r = self.locate(cursor_offset, line)
         if r is None:
@@ -365,8 +432,9 @@ class DictKeyCompletion(BaseCompletionType):
         except EvaluationError:
             return None
         if isinstance(obj, dict) and obj.keys():
-            matches = set("{0!r}]".format(k) for k in obj.keys()
-                          if repr(k).startswith(r.word))
+            matches = {
+                f"{k!r}]" for k in obj.keys() if repr(k).startswith(r.word)
+            }
             return matches if matches else None
         else:
             return None
@@ -379,33 +447,31 @@ class DictKeyCompletion(BaseCompletionType):
 
 
 class MagicMethodCompletion(BaseCompletionType):
-
     def matches(self, cursor_offset, line, **kwargs):
-        if 'current_block' not in kwargs:
+        if "current_block" not in kwargs:
             return None
-        current_block = kwargs['current_block']
+        current_block = kwargs["current_block"]
 
         r = self.locate(cursor_offset, line)
         if r is None:
             return None
-        if 'class' not in current_block:
+        if "class" not in current_block:
             return None
-        return set(name for name in MAGIC_METHODS if name.startswith(r.word))
+        return {name for name in MAGIC_METHODS if name.startswith(r.word)}
 
     def locate(self, current_offset, line):
         return lineparts.current_method_definition_name(current_offset, line)
 
 
 class GlobalCompletion(BaseCompletionType):
-
     def matches(self, cursor_offset, line, **kwargs):
         """Compute matches when text is a simple name.
         Return a list of all keywords, built-in functions and names currently
         defined in self.namespace that match.
         """
-        if 'locals_' not in kwargs:
+        if "locals_" not in kwargs:
             return None
-        locals_ = kwargs['locals_']
+        locals_ = kwargs["locals_"]
 
         r = self.locate(cursor_offset, line)
         if r is None:
@@ -417,13 +483,14 @@ class GlobalCompletion(BaseCompletionType):
             if self.method_match(word, n, r.word):
                 matches.add(word)
         for nspace in (builtins.__dict__, locals_):
-            for word, val in iteritems(nspace):
-                word = try_decode(word, 'ascii')
+            for word, val in nspace.items():
                 # if identifier isn't ascii, don't complete (syntax error)
                 if word is None:
                     continue
-                if (self.method_match(word, n, r.word) and
-                        word != "__builtins__"):
+                if (
+                    self.method_match(word, n, r.word)
+                    and word != "__builtins__"
+                ):
                     matches.add(_callable_postfix(val, word))
         return matches if matches else None
 
@@ -432,11 +499,10 @@ class GlobalCompletion(BaseCompletionType):
 
 
 class ParameterNameCompletion(BaseCompletionType):
-
     def matches(self, cursor_offset, line, **kwargs):
-        if 'argspec' not in kwargs:
+        if "argspec" not in kwargs:
             return None
-        argspec = kwargs['argspec']
+        argspec = kwargs["argspec"]
 
         if not argspec:
             return None
@@ -444,12 +510,14 @@ class ParameterNameCompletion(BaseCompletionType):
         if r is None:
             return None
         if argspec:
-            matches = set(name + '=' for name in argspec[1][0]
-                          if isinstance(name, string_types) and
-                          name.startswith(r.word))
-            if py3:
-                matches.update(name + '=' for name in argspec[1][4]
-                               if name.startswith(r.word))
+            matches = {
+                name + "="
+                for name in argspec[1][0]
+                if isinstance(name, str) and name.startswith(r.word)
+            }
+            matches.update(
+                name + "=" for name in argspec[1][4] if name.startswith(r.word)
+            )
         return matches if matches else None
 
     def locate(self, current_offset, line):
@@ -462,9 +530,9 @@ class ExpressionAttributeCompletion(AttrCompletion):
         return lineparts.current_expression_attribute(current_offset, line)
 
     def matches(self, cursor_offset, line, **kwargs):
-        if 'locals_' not in kwargs:
+        if "locals_" not in kwargs:
             return None
-        locals_ = kwargs['locals_']
+        locals_ = kwargs["locals_"]
 
         if locals_ is None:
             locals_ = __main__.__dict__
@@ -475,35 +543,38 @@ class ExpressionAttributeCompletion(AttrCompletion):
             obj = evaluate_current_expression(cursor_offset, line, locals_)
         except EvaluationError:
             return set()
-        with inspection.AttrCleaner(obj):
-            #          strips leading dot
-            matches = [m[1:] for m in self.attr_lookup(obj, '', attr.word)]
 
-        return set(m for m in matches if few_enough_underscores(attr.word, m))
+        #        strips leading dot
+        matches = [m[1:] for m in self.attr_lookup(obj, "", attr.word)]
+        return {m for m in matches if few_enough_underscores(attr.word, m)}
 
 
 try:
     import jedi
 except ImportError:
+
     class MultilineJediCompletion(BaseCompletionType):
         def matches(self, cursor_offset, line, **kwargs):
             return None
-else:
-    class JediCompletion(BaseCompletionType):
 
+
+else:
+
+    class JediCompletion(BaseCompletionType):
         def matches(self, cursor_offset, line, **kwargs):
-            if 'history' not in kwargs:
+            if "history" not in kwargs:
                 return None
-            history = kwargs['history']
+            history = kwargs["history"]
 
             if not lineparts.current_word(cursor_offset, line):
                 return None
-            history = '\n'.join(history) + '\n' + line
+            history = "\n".join(history) + "\n" + line
 
             try:
-                script = jedi.Script(history, len(history.splitlines()),
-                                     cursor_offset, 'fake.py')
-                completions = script.completions()
+                script = jedi.Script(history, path="fake.py")
+                completions = script.complete(
+                    len(history.splitlines()), cursor_offset
+                )
             except (jedi.NotFoundError, IndexError, KeyError):
                 # IndexError for #483
                 # KeyError for #544
@@ -517,17 +588,18 @@ else:
                 self._orig_start = None
                 return None
 
-            first_letter = line[self._orig_start:self._orig_start + 1]
+            first_letter = line[self._orig_start : self._orig_start + 1]
 
             matches = [c.name for c in completions]
-            if any(not m.lower().startswith(matches[0][0].lower())
-                   for m in matches):
+            if any(
+                not m.lower().startswith(matches[0][0].lower()) for m in matches
+            ):
                 # Too general - giving completions starting with multiple
                 # letters
                 return None
             else:
                 # case-sensitive matches only
-                return set(m for m in matches if m.startswith(first_letter))
+                return {m for m in matches if m.startswith(first_letter)}
 
         def locate(self, cursor_offset, line):
             start = self._orig_start
@@ -536,17 +608,17 @@ else:
 
     class MultilineJediCompletion(JediCompletion):
         def matches(self, cursor_offset, line, **kwargs):
-            if 'current_block' not in kwargs or 'history' not in kwargs:
+            if "current_block" not in kwargs or "history" not in kwargs:
                 return None
-            current_block = kwargs['current_block']
-            history = kwargs['history']
+            current_block = kwargs["current_block"]
+            history = kwargs["history"]
 
-            if '\n' in current_block:
-                assert cursor_offset <= len(line), "%r %r" % (cursor_offset,
-                                                              line)
-                results = super(MultilineJediCompletion,
-                                self).matches(cursor_offset, line,
-                                              history=history)
+            if "\n" in current_block:
+                assert cursor_offset <= len(line), "{!r} {!r}".format(
+                    cursor_offset,
+                    line,
+                )
+                results = super().matches(cursor_offset, line, history=history)
                 return results
             else:
                 return None
@@ -576,8 +648,10 @@ def get_completer(completers, cursor_offset, line, **kwargs):
             # Instead of crashing the UI, log exceptions from autocompleters.
             logger = logging.getLogger(__name__)
             logger.debug(
-                'Completer {} failed with unhandled exception: {}'.format(
-                    completer, e))
+                "Completer {} failed with unhandled exception: {}".format(
+                    completer, e
+                )
+            )
             continue
         if matches is not None:
             return sorted(matches), (completer if matches else None)
@@ -585,30 +659,24 @@ def get_completer(completers, cursor_offset, line, **kwargs):
     return [], None
 
 
-def get_default_completer(mode=SIMPLE):
+def get_default_completer(mode=AutocompleteModes.SIMPLE, module_gatherer=None):
     return (
         DictKeyCompletion(mode=mode),
-        ImportCompletion(mode=mode),
+        ImportCompletion(module_gatherer, mode=mode),
         FilenameCompletion(mode=mode),
         MagicMethodCompletion(mode=mode),
         MultilineJediCompletion(mode=mode),
-        CumulativeCompleter((GlobalCompletion(mode=mode),
-                             ParameterNameCompletion(mode=mode)),
-                            mode=mode),
+        CumulativeCompleter(
+            (GlobalCompletion(mode=mode), ParameterNameCompletion(mode=mode)),
+            mode=mode,
+        ),
         AttrCompletion(mode=mode),
         ExpressionAttributeCompletion(mode=mode),
     )
 
 
-def get_completer_bpython(cursor_offset, line, **kwargs):
-    """"""
-    return get_completer(get_default_completer(),
-                         cursor_offset, line, **kwargs)
-
-
 def _callable_postfix(value, word):
     """rlcompleter's _callable_postfix done right."""
-    with inspection.AttrCleaner(value):
-        if inspection.is_callable(value):
-            word += '('
+    if callable(value):
+        word += "("
     return word
