@@ -39,7 +39,6 @@
 # - Instead the suspend key exits the program
 # - View source doesn't work on windows unless you install the less program (From GnuUtils or Cygwin)
 
-from __future__ import division, absolute_import
 
 import platform
 import os
@@ -59,18 +58,14 @@ if platform.system() != "Windows":
 import unicodedata
 import errno
 
-from six.moves import range
 
 # These are used for syntax highlighting
 from pygments import format
 from pygments.formatters import TerminalFormatter
-from ._py3compat import PythonLexer
+from pygments.lexers import Python3Lexer
 from pygments.token import Token
 from .formatter import BPythonFormatter
 
-# This for completion
-from . import importcompletion
-
 # This for config
 from .config import Struct, getpreferredencoding
 
@@ -83,13 +78,9 @@ from .translations import _
 
 from . import repl
 from . import args as bpargs
-from ._py3compat import py3
 from .pager import page
 from .args import parse as argsparse
 
-if not py3:
-    import inspect
-
 
 # --- module globals ---
 stdscr = None
@@ -127,7 +118,7 @@ def forward_if_not_current(func):
     return newfunc
 
 
-class FakeStream(object):
+class FakeStream:
     """Provide a fake file object which calls functions on the interface
     provided."""
 
@@ -153,7 +144,7 @@ class FakeStream(object):
         self.interface.flush()
 
 
-class FakeStdin(object):
+class FakeStdin:
     """Provide a fake stdin type for things like raw_input() etc."""
 
     def __init__(self, interface):
@@ -174,7 +165,7 @@ class FakeStdin(object):
     def write(self, value):
         # XXX IPython expects sys.stdin.write to exist, there will no doubt be
         # others, so here's a hack to keep them happy
-        raise IOError(errno.EBADF, "sys.stdin is read-only")
+        raise OSError(errno.EBADF, "sys.stdin is read-only")
 
     def isatty(self):
         return True
@@ -196,7 +187,7 @@ class FakeStdin(object):
         try:
             while not buffer.endswith(("\n", "\r")):
                 key = self.interface.get_key()
-                if key in [curses.erasechar(), "KEY_BACKSPACE"]:
+                if key in (curses.erasechar(), "KEY_BACKSPACE"):
                     y, x = self.interface.scr.getyx()
                     if buffer:
                         self.interface.scr.delch(y, x - 1)
@@ -222,10 +213,7 @@ class FakeStdin(object):
                 self.buffer.append(rest)
             buffer = buffer[:size]
 
-        if py3:
-            return buffer
-        else:
-            return buffer.encode(getpreferredencoding())
+        return buffer
 
     def read(self, size=None):
         if size == 0:
@@ -312,7 +300,7 @@ def make_colors(config):
 
 class CLIInteraction(repl.Interaction):
     def __init__(self, config, statusbar=None):
-        super(CLIInteraction, self).__init__(config, statusbar)
+        super().__init__(config, statusbar)
 
     def confirm(self, q):
         """Ask for yes or no and return boolean"""
@@ -332,7 +320,7 @@ class CLIInteraction(repl.Interaction):
 
 class CLIRepl(repl.Repl):
     def __init__(self, scr, interp, statusbar, config, idle=None):
-        super(CLIRepl, self).__init__(interp, config)
+        super().__init__(interp, config)
         self.interp.writetb = self.writetb
         self.scr = scr
         self.stdout_hist = ""  # native str (bytes in Py2, unicode in Py3)
@@ -536,11 +524,8 @@ class CLIRepl(repl.Repl):
     def echo(self, s, redraw=True):
         """Parse and echo a formatted string with appropriate attributes. It
         uses the formatting method as defined in formatter.py to parse the
-        srings. It won't update the screen if it's reevaluating the code (as it
+        strings. It won't update the screen if it's reevaluating the code (as it
         does with undo)."""
-        if not py3 and isinstance(s, unicode):
-            s = s.encode(getpreferredencoding())
-
         a = get_colpair(self.config, "output")
         if "\x01" in s:
             rx = re.search("\x01([A-Za-z])([A-Za-z]?)", s)
@@ -631,14 +616,11 @@ class CLIRepl(repl.Repl):
         while True:
             try:
                 key += self.scr.getkey()
-                if py3:
-                    # Seems like we get a in the locale's encoding
-                    # encoded string in Python 3 as well, but of
-                    # type str instead of bytes, hence convert it to
-                    # bytes first and decode then
-                    key = key.encode("latin-1").decode(getpreferredencoding())
-                else:
-                    key = key.decode(getpreferredencoding())
+                # Seems like we get a in the locale's encoding
+                # encoded string in Python 3 as well, but of
+                # type str instead of bytes, hence convert it to
+                # bytes first and decode then
+                key = key.encode("latin-1").decode(getpreferredencoding())
                 self.scr.nodelay(False)
             except UnicodeDecodeError:
                 # Yes, that actually kind of sucks, but I don't see another way to get
@@ -728,9 +710,8 @@ class CLIRepl(repl.Repl):
         _args = topline.argspec.varargs
         _kwargs = topline.argspec.varkwargs
         is_bound_method = topline.is_bound_method
-        if py3:
-            kwonly = topline.argspec.kwonly
-            kwonly_defaults = topline.argspec.kwonly_defaults or dict()
+        kwonly = topline.argspec.kwonly
+        kwonly_defaults = topline.argspec.kwonly_defaults or dict()
         max_w = int(self.scr.getmaxyx()[1] * 0.6)
         self.list_win.erase()
         self.list_win.resize(3, max_w)
@@ -778,13 +759,7 @@ class CLIRepl(repl.Repl):
             if k == in_arg or i == in_arg:
                 color |= curses.A_BOLD
 
-            if not py3:
-                # See issue #138: We need to format tuple unpacking correctly
-                # We use the undocumented function inspection.strseq() for
-                # that. Fortunately, that madness is gone in Python 3.
-                self.list_win.addstr(inspect.strseq(i, str), color)
-            else:
-                self.list_win.addstr(str(i), color)
+            self.list_win.addstr(str(i), color)
             if kw is not None:
                 self.list_win.addstr("=", punctuation_colpair)
                 self.list_win.addstr(kw, get_colpair(self.config, "token"))
@@ -794,11 +769,9 @@ class CLIRepl(repl.Repl):
         if _args:
             if args:
                 self.list_win.addstr(", ", punctuation_colpair)
-            self.list_win.addstr(
-                "*%s" % (_args,), get_colpair(self.config, "token")
-            )
+            self.list_win.addstr(f"*{_args}", get_colpair(self.config, "token"))
 
-        if py3 and kwonly:
+        if kwonly:
             if not _args:
                 if args:
                     self.list_win.addstr(", ", punctuation_colpair)
@@ -818,10 +791,10 @@ class CLIRepl(repl.Repl):
                     )
 
         if _kwargs:
-            if args or _args or (py3 and kwonly):
+            if args or _args or kwonly:
                 self.list_win.addstr(", ", punctuation_colpair)
             self.list_win.addstr(
-                "**%s" % (_kwargs,), get_colpair(self.config, "token")
+                f"**{_kwargs}", get_colpair(self.config, "token")
             )
         self.list_win.addstr(")", punctuation_colpair)
 
@@ -940,11 +913,11 @@ class CLIRepl(repl.Repl):
             # Redraw (as there might have been highlighted parens)
             self.print_line(self.s)
 
-        elif key in ("KEY_NPAGE", "\T"):  # page_down or \T
+        elif key in ("KEY_NPAGE",):  # page_down
             self.hend()
             self.print_line(self.s)
 
-        elif key in ("KEY_PPAGE", "\S"):  # page_up or \S
+        elif key in ("KEY_PPAGE",):  # page_up
             self.hbegin()
             self.print_line(self.s)
 
@@ -967,7 +940,8 @@ class CLIRepl(repl.Repl):
             return ""
 
         elif key in key_dispatch[config.clear_screen_key]:
-            self.s_hist = [self.s_hist[-1]]
+            # clear all but current line
+            self.screen_hist = [self.screen_hist[-1]]
             self.highlighted_paren = None
             self.redraw()
             return ""
@@ -999,11 +973,11 @@ class CLIRepl(repl.Repl):
             try:
                 source = self.get_source_of_current_name()
             except repl.SourceNotFound as e:
-                self.statusbar.message("%s" % (e,))
+                self.statusbar.message(f"{e}")
             else:
                 if config.highlight_show_source:
                     source = format(
-                        PythonLexer().get_tokens(source), TerminalFormatter()
+                        Python3Lexer().get_tokens(source), TerminalFormatter()
                     )
                 page(source)
             return ""
@@ -1093,25 +1067,21 @@ class CLIRepl(repl.Repl):
         """Show the appropriate Python prompt"""
         if not more:
             self.echo(
-                "\x01%s\x03%s" % (self.config.color_scheme["prompt"], self.ps1)
+                "\x01{}\x03{}".format(
+                    self.config.color_scheme["prompt"], self.ps1
+                )
             )
-            if py3:
-                self.stdout_hist += self.ps1
-            else:
-                self.stdout_hist += self.ps1.encode(getpreferredencoding())
-            self.s_hist.append(
+            self.stdout_hist += self.ps1
+            self.screen_hist.append(
                 "\x01%s\x03%s\x04"
                 % (self.config.color_scheme["prompt"], self.ps1)
             )
         else:
             prompt_more_color = self.config.color_scheme["prompt_more"]
-            self.echo("\x01%s\x03%s" % (prompt_more_color, self.ps2))
-            if py3:
-                self.stdout_hist += self.ps2
-            else:
-                self.stdout_hist += self.ps2.encode(getpreferredencoding())
-            self.s_hist.append(
-                "\x01%s\x03%s\x04" % (prompt_more_color, self.ps2)
+            self.echo(f"\x01{prompt_more_color}\x03{self.ps2}")
+            self.stdout_hist += self.ps2
+            self.screen_hist.append(
+                f"\x01{prompt_more_color}\x03{self.ps2}\x04"
             )
 
     def push(self, s, insert_into_history=True):
@@ -1128,15 +1098,15 @@ class CLIRepl(repl.Repl):
             curses.raw(True)
 
     def redraw(self):
-        """Redraw the screen."""
+        """Redraw the screen using screen_hist"""
         self.scr.erase()
-        for k, s in enumerate(self.s_hist):
+        for k, s in enumerate(self.screen_hist):
             if not s:
                 continue
             self.iy, self.ix = self.scr.getyx()
             for i in s.split("\x04"):
                 self.echo(i, redraw=False)
-            if k < len(self.s_hist) - 1:
+            if k < len(self.screen_hist) - 1:
                 self.scr.addstr("\n")
         self.iy, self.ix = self.scr.getyx()
         self.print_line(self.s)
@@ -1175,11 +1145,8 @@ class CLIRepl(repl.Repl):
                 return self.exit_value
 
             self.history.append(inp)
-            self.s_hist[-1] += self.f_string
-            if py3:
-                self.stdout_hist += inp + "\n"
-            else:
-                self.stdout_hist += inp.encode(getpreferredencoding()) + "\n"
+            self.screen_hist[-1] += self.f_string
+            self.stdout_hist += inp + "\n"
             stdout_position = len(self.stdout_hist)
             self.more = self.push(inp)
             if not self.more:
@@ -1234,7 +1201,7 @@ class CLIRepl(repl.Repl):
         self.f_string = ""
         self.buffer = []
         self.scr.erase()
-        self.s_hist = []
+        self.screen_hist = []
         # Set cursor position to -1 to prevent paren matching
         self.cpos = -1
 
@@ -1242,12 +1209,9 @@ class CLIRepl(repl.Repl):
 
         self.iy, self.ix = self.scr.getyx()
         for line in self.history:
-            if py3:
-                self.stdout_hist += line + "\n"
-            else:
-                self.stdout_hist += line.encode(getpreferredencoding()) + "\n"
+            self.stdout_hist += line + "\n"
             self.print_line(line)
-            self.s_hist[-1] += self.f_string
+            self.screen_hist[-1] += self.f_string
             # I decided it was easier to just do this manually
             # than to make the print_line and history stuff more flexible.
             self.scr.addstr("\n")
@@ -1279,16 +1243,13 @@ class CLIRepl(repl.Repl):
         else:
             t = s
 
-        if not py3 and isinstance(t, unicode):
-            t = t.encode(getpreferredencoding())
-
         if not self.stdout_hist:
             self.stdout_hist = t
         else:
             self.stdout_hist += t
 
         self.echo(s)
-        self.s_hist.append(s.rstrip())
+        self.screen_hist.append(s.rstrip())
 
     def show_list(
         self, items, arg_pos, topline=None, formatter=None, current_item=None
@@ -1391,16 +1352,12 @@ class CLIRepl(repl.Repl):
         if v_items:
             self.list_win.addstr("\n ")
 
-        if not py3:
-            encoding = getpreferredencoding()
         for ix, i in enumerate(v_items):
             padding = (wl - len(i)) * " "
             if i == current_item:
                 color = get_colpair(self.config, "operator")
             else:
                 color = get_colpair(self.config, "main")
-            if not py3:
-                i = i.encode(encoding)
             self.list_win.addstr(i + padding, color)
             if (cols == 1 or (ix and not (ix + 1) % cols)) and ix + 1 < len(
                 v_items
@@ -1408,8 +1365,6 @@ class CLIRepl(repl.Repl):
                 self.list_win.addstr("\n ")
 
         if self.docstring is not None:
-            if not py3 and isinstance(docstring_string, unicode):
-                docstring_string = docstring_string.encode(encoding, "ignore")
             self.list_win.addstr(
                 "\n" + docstring_string, get_colpair(self.config, "comment")
             )
@@ -1522,7 +1477,7 @@ class CLIRepl(repl.Repl):
     def writetb(self, lines):
         for line in lines:
             self.write(
-                "\x01%s\x03%s" % (self.config.color_scheme["error"], line)
+                "\x01{}\x03{}".format(self.config.color_scheme["error"], line)
             )
 
     def yank_from_buffer(self):
@@ -1545,13 +1500,10 @@ class CLIRepl(repl.Repl):
         self.iy, self.ix = self.scr.getyx()
         self.evaluating = True
         for line in lines:
-            if py3:
-                self.stdout_hist += line + "\n"
-            else:
-                self.stdout_hist += line.encode(getpreferredencoding()) + "\n"
+            self.stdout_hist += line + "\n"
             self.history.append(line)
             self.print_line(line)
-            self.s_hist[-1] += self.f_string
+            self.screen_hist[-1] += self.f_string
             self.scr.addstr("\n")
             self.more = self.push(line)
             self.prompt(self.more)
@@ -1572,7 +1524,7 @@ class CLIRepl(repl.Repl):
         return ""
 
 
-class Statusbar(object):
+class Statusbar:
     """This class provides the status bar at the bottom of the screen.
     It has message() and prompt() methods for user interactivity, as
     well as settext() and clear() methods for changing its appearance.
@@ -1706,9 +1658,6 @@ class Statusbar(object):
             self.c = c
 
         if s:
-            if not py3 and isinstance(s, unicode):
-                s = s.encode(getpreferredencoding())
-
             if self.c:
                 self.win.addstr(s, self.c)
             else:
@@ -1748,7 +1697,7 @@ def init_wins(scr, config):
     )
 
     message = "  ".join(
-        "<%s> %s" % (key, command) for command, key in commands if key
+        f"<{key}> {command}" for command, key in commands if key
     )
 
     statusbar = Statusbar(
@@ -1832,7 +1781,7 @@ def idle(caller):
     sure it happens conveniently."""
     global DO_RESIZE
 
-    if importcompletion.find_coroutine() or caller.paste_mode:
+    if caller.module_gatherer.find_coroutine() or caller.paste_mode:
         caller.scr.nodelay(True)
         key = caller.scr.getch()
         caller.scr.nodelay(False)
@@ -1869,7 +1818,7 @@ def do_resize(caller):
     # The list win resizes itself every time it appears so no need to do it here.
 
 
-class FakeDict(object):
+class FakeDict:
     """Very simple dict-alike that returns a constant value for any key -
     used as a hacky solution to using a colours dict containing colour codes if
     colour initialisation fails."""
@@ -1997,15 +1946,6 @@ def main_curses(scr, args, config, interactive=True, locals_=None, banner=None):
     )
     clirepl.write("\n")
 
-    if sys.version_info[0] == 2:
-        # XXX these deprecation warnings need to go at some point
-        clirepl.write(
-            _(
-                "WARNING: You are using `bpython` on Python 2. Support for Python 2 has been deprecated in version 0.19 and might disappear in a future version."
-            )
-        )
-        clirepl.write("\n")
-
     exit_value = clirepl.repl()
     if hasattr(sys, "exitfunc"):
         sys.exitfunc()
