@@ -1,12 +1,6 @@
-# encoding: utf-8
-
-from __future__ import absolute_import
-
 import collections
-import io
 import logging
 import sys
-from optparse import Option
 
 import curtsies
 import curtsies.window
@@ -19,19 +13,13 @@ from .curtsiesfrontend.interpreter import Interp
 from . import args as bpargs
 from . import translations
 from .translations import _
-from .importcompletion import find_iterator
 from .curtsiesfrontend import events as bpythonevents
 from . import inspection
 from .repl import extract_exit_value
-from ._py3compat import py3
 
 logger = logging.getLogger(__name__)
 
 
-repl = None  # global for `from bpython.curtsies import repl`
-# WARNING Will be a problem if more than one repl is ever instantiated this way
-
-
 class FullCurtsiesRepl(BaseRepl):
     def __init__(self, config, locals_, banner, interp=None):
         self.input_generator = curtsies.input.Input(
@@ -54,8 +42,8 @@ class FullCurtsiesRepl(BaseRepl):
         self._request_reload = self.input_generator.threadsafe_event_trigger(
             bpythonevents.ReloadEvent
         )
-        self.interrupting_refresh = self.input_generator.threadsafe_event_trigger(
-            lambda: None
+        self.interrupting_refresh = (
+            self.input_generator.threadsafe_event_trigger(lambda: None)
         )
         self.request_undo = self.input_generator.event_trigger(
             bpythonevents.UndoEvent
@@ -64,9 +52,9 @@ class FullCurtsiesRepl(BaseRepl):
         with self.input_generator:
             pass  # temp hack to get .original_stty
 
-        super(FullCurtsiesRepl, self).__init__(
+        super().__init__(
+            config,
             locals_=locals_,
-            config=config,
             banner=banner,
             interp=interp,
             orig_tcattrs=self.input_generator.original_stty,
@@ -124,7 +112,7 @@ class FullCurtsiesRepl(BaseRepl):
         # do a display before waiting for first event
         self.process_event_and_paint(None)
         inputs = combined_events(self.input_generator)
-        for unused in find_iterator:
+        while self.module_gatherer.find_coroutine():
             e = inputs.send(0)
             if e is not None:
                 self.process_event_and_paint(e)
@@ -140,30 +128,31 @@ def main(args=None, locals_=None, banner=None, welcome_message=None):
     """
     translations.init()
 
+    def curtsies_arguments(parser):
+        parser.add_argument(
+            "--log",
+            "-L",
+            action="count",
+            help=_("log debug messages to bpython.log"),
+        )
+        parser.add_argument(
+            "--paste",
+            "-p",
+            action="store_true",
+            help=_("start by pasting lines of a file into session"),
+        )
+
     config, options, exec_args = bpargs.parse(
         args,
         (
-            "curtsies options",
-            None,
-            [
-                Option(
-                    "--log",
-                    "-L",
-                    action="count",
-                    help=_("log debug messages to bpython.log"),
-                ),
-                Option(
-                    "--paste",
-                    "-p",
-                    action="store_true",
-                    help=_("start by pasting lines of a file into session"),
-                ),
-            ],
+            _("curtsies arguments"),
+            _("Additional arguments specific to the curtsies-based REPL."),
+            curtsies_arguments,
         ),
     )
     if options.log is None:
         options.log = 0
-    logging_levels = [logging.ERROR, logging.INFO, logging.DEBUG]
+    logging_levels = (logging.ERROR, logging.INFO, logging.DEBUG)
     level = logging_levels[min(len(logging_levels) - 1, options.log)]
     logging.getLogger("curtsies").setLevel(level)
     logging.getLogger("bpython").setLevel(level)
@@ -183,7 +172,7 @@ def main(args=None, locals_=None, banner=None, welcome_message=None):
         if options.paste:
             paste = curtsies.events.PasteEvent()
             encoding = inspection.get_encoding_file(exec_args[0])
-            with io.open(exec_args[0], encoding=encoding) as f:
+            with open(exec_args[0], encoding=encoding) as f:
                 sourcecode = f.read()
             paste.events.extend(sourcecode)
         else:
@@ -203,15 +192,6 @@ def main(args=None, locals_=None, banner=None, welcome_message=None):
     if banner is not None:
         print(banner)
 
-    if not py3:
-        # XXX these deprecation warnings need to go at some point
-        print(
-            _(
-                "WARNING: You are using `bpython` on Python 2. Support for Python 2 has been deprecated in version 0.19 and might disappear in a future version."
-            )
-        )
-
-    global repl
     repl = FullCurtsiesRepl(config, locals_, welcome_message, interp)
     try:
         with repl.input_generator:
