@@ -1,5 +1,3 @@
-# encoding: utf-8
-
 # The MIT License
 #
 # Copyright (c) 2015 the bpython authors.
@@ -28,25 +26,17 @@ In order to provide fancy completion, some code can be executed safely.
 
 """
 
-from __future__ import absolute_import
 
 import ast
-import inspect
-from six import string_types
-from six.moves import builtins
+import sys
+import builtins
 
 from . import line as line_properties
-from ._py3compat import py3
-from .inspection import is_new_style, AttrCleaner
+from .inspection import getattr_safe
 
-_string_type_nodes = (ast.Str, ast.Bytes) if py3 else (ast.Str,)
-_numeric_types = (int, float, complex) + (() if py3 else (long,))
-
-# added in Python 3.4
-if hasattr(ast, "NameConstant"):
-    _name_type_nodes = (ast.Name, ast.NameConstant)
-else:
-    _name_type_nodes = (ast.Name,)
+_string_type_nodes = (ast.Str, ast.Bytes)
+_numeric_types = (int, float, complex)
+_name_type_nodes = (ast.Name, ast.NameConstant)
 
 
 class EvaluationError(Exception):
@@ -65,9 +55,8 @@ def safe_eval(expr, namespace):
 
 # This function is under the Python License, Version 2
 # This license requires modifications to the code be reported.
-# Based on ast.literal_eval in Python 2 and Python 3
+# Based on ast.literal_eval
 # Modifications:
-# * Python 2 and Python 3 versions of the function are combined
 # * checks that objects used as operands of + and - are numbers
 #   instead of checking they are constructed with number literals
 # * new docstring describing different functionality
@@ -80,24 +69,26 @@ def simple_eval(node_or_string, namespace=None):
 
     The string or node provided may only consist of:
     * the following Python literal structures: strings, numbers, tuples,
-        lists, and dicts
+        lists, dicts, and sets
     * variable names causing lookups in the passed in namespace or builtins
     * getitem calls using the [] syntax on objects of the types above
 
-    Like the Python 3 (and unlike the Python 2) literal_eval, unary and binary
-    + and - operations are allowed on all builtin numeric types.
+    Like Python 3's literal_eval, unary and binary + and - operations are
+    allowed on all builtin numeric types.
 
-    The optional namespace dict-like ought not to cause side effects on lookup
+    The optional namespace dict-like ought not to cause side effects on lookup.
     """
     if namespace is None:
         namespace = {}
-    if isinstance(node_or_string, string_types):
+    if isinstance(node_or_string, str):
         node_or_string = ast.parse(node_or_string, mode="eval")
     if isinstance(node_or_string, ast.Expression):
         node_or_string = node_or_string.body
 
     def _convert(node):
-        if isinstance(node, _string_type_nodes):
+        if isinstance(node, ast.Constant):
+            return node.value
+        elif isinstance(node, _string_type_nodes):
             return node.s
         elif isinstance(node, ast.Num):
             return node.n
@@ -106,10 +97,18 @@ def simple_eval(node_or_string, namespace=None):
         elif isinstance(node, ast.List):
             return list(map(_convert, node.elts))
         elif isinstance(node, ast.Dict):
-            return dict(
-                (_convert(k), _convert(v))
-                for k, v in zip(node.keys, node.values)
-            )
+            return {
+                _convert(k): _convert(v) for k, v in zip(node.keys, node.values)
+            }
+        elif isinstance(node, ast.Set):
+            return set(map(_convert, node.elts))
+        elif (
+            isinstance(node, ast.Call)
+            and isinstance(node.func, ast.Name)
+            and node.func.id == "set"
+            and node.args == node.keywords == []
+        ):
+            return set()
 
         # this is a deviation from literal_eval: we allow non-literals
         elif isinstance(node, _name_type_nodes):
@@ -140,7 +139,8 @@ def simple_eval(node_or_string, namespace=None):
             left = _convert(node.left)
             right = _convert(node.right)
             if not (
-                type(left) in _numeric_types and type(right) in _numeric_types
+                isinstance(left, _numeric_types)
+                and isinstance(right, _numeric_types)
             ):
                 raise ValueError("binary + and - only allowed on builtin nums")
             if isinstance(node.op, ast.Add):
@@ -155,25 +155,34 @@ def simple_eval(node_or_string, namespace=None):
             obj = _convert(node.value)
             index = _convert(node.slice.value)
             return safe_getitem(obj, index)
+        elif (
+            sys.version_info[:2] >= (3, 9)
+            and isinstance(node, ast.Subscript)
+            and isinstance(node.slice, (ast.Constant, ast.Name))
+        ):
+            obj = _convert(node.value)
+            index = _convert(node.slice)
+            return safe_getitem(obj, index)
 
         # this is a deviation from literal_eval: we allow attribute access
         if isinstance(node, ast.Attribute):
             obj = _convert(node.value)
             attr = node.attr
-            return safe_get_attribute(obj, attr)
+            return getattr_safe(obj, attr)
 
-        raise ValueError("malformed string")
+        raise ValueError(f"malformed node or string: {node!r}")
 
     return _convert(node_or_string)
 
 
 def safe_getitem(obj, index):
-    if type(obj) in (list, tuple, dict, bytes) + string_types:
+    """ Safely tries to access obj[index] """
+    if type(obj) in (list, tuple, dict, bytes, str):
         try:
             return obj[index]
         except (KeyError, IndexError):
-            raise EvaluationError("can't lookup key %r on %r" % (index, obj))
-    raise ValueError("unsafe to lookup on object of type %s" % (type(obj),))
+            raise EvaluationError(f"can't lookup key {index!r} on {obj!r}")
+    raise ValueError("unsafe to lookup on object of type {}".format(type(obj)))
 
 
 def find_attribute_with_name(node, name):
@@ -244,48 +253,4 @@ def evaluate_current_attribute(cursor_offset, line, namespace=None):
     try:
         return getattr(obj, attr.word)
     except AttributeError:
-        raise EvaluationError(
-            "can't lookup attribute %s on %r" % (attr.word, obj)
-        )
-
-
-def safe_get_attribute(obj, attr):
-    """Gets attributes without triggering descriptors on new-style classes"""
-    if is_new_style(obj):
-        with AttrCleaner(obj):
-            result = safe_get_attribute_new_style(obj, attr)
-            if isinstance(result, member_descriptor):
-                # will either be the same slot descriptor or the value
-                return getattr(obj, attr)
-            return result
-    return getattr(obj, attr)
-
-
-class _ClassWithSlots(object):
-    __slots__ = ["a"]
-
-
-member_descriptor = type(_ClassWithSlots.a)
-
-
-def safe_get_attribute_new_style(obj, attr):
-    """Returns approximately the attribute returned by getattr(obj, attr)
-
-    The object returned ought to be callable if getattr(obj, attr) was.
-    Fake callable objects may be returned instead, in order to avoid executing
-    arbitrary code in descriptors.
-
-    If the object is an instance of a class that uses __slots__, will return
-    the member_descriptor object instead of the value.
-    """
-    if not is_new_style(obj):
-        raise ValueError("%r is not a new-style class or object" % obj)
-    to_look_through = (
-        obj.__mro__ if inspect.isclass(obj) else (obj,) + type(obj).__mro__
-    )
-
-    for cls in to_look_through:
-        if hasattr(cls, "__dict__") and attr in cls.__dict__:
-            return cls.__dict__[attr]
-
-    raise AttributeError()
+        raise EvaluationError(f"can't lookup attribute {attr.word} on {obj!r}")
